/*****************************************************************************/
/**                                                                         **/
/**                 catalog v. 1.0 directory/file catalog utility           **/
/**                     Copyright Jan Marco Alkema, 2002                    **/
/**                                                                         **/
/**                               catalog.c                                 **/
/*****************************************************************************/
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdarg.h>
#include <ctype.h>
#include <malloc.h>
#include <fcntl.h>
#include <math.h>
#include <mysql.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>

#include "md5.h"
#include "catalog.h"

  MYSQL *m_mysql;
  char m_pszHost[50] = "localhost";
  char m_pszUser[50] = "root";
  char m_pszPasswd[50] = "";
  char m_pszDb[50] = "gnunet";
  unsigned m_nPort = 0;
  char *m_pszUnixSocket = NULL;
  unsigned nClientFlag = 0; 
  char query[4096] = "";

  MYSQL_FIELD *field;
  MYSQL_ROW row;
  MYSQL_RES *result;
  unsigned long *lengthsi, file_size_not_insert = 0, number_of_not_inserted_files = 0;
  unsigned int num_fields, off;
  unsigned int i;
  FILE *input, *output, *output2;

/*
 * This file builds an executable that performs various functions related
 * to the MD5 library.  Typical compilation:
 *	gcc -o md5main -lm md5main.c md5.c
 */
static const char *const usage = "\
Usage:\n\
    md5main --test		# run the self-test (A.5 of RFC 1321)\n\
    md5main --t-values		# print the T values for the library\n\
    md5main --version		# print the version of the package\n\
";
static const char *const version = "2002-04-13";

/* Run the self-test. */
static int
do_test(void)
{
    static const char *const test[7*2] = {
	"", "d41d8cd98f00b204e9800998ecf8427e",
	"a", "0cc175b9c0f1b6a831c399e269772661",
	"abc", "900150983cd24fb0d6963f7d28e17f72",
	"message digest", "f96b697d7cb7938d525a2f31aaf161d0",
	"abcdefghijklmnopqrstuvwxyz", "c3fcd3d76192e4007dfb496cca67e13b",
	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789",
				"d174ab98d277d9f5a5611c2c9f419d9f",
	"12345678901234567890123456789012345678901234567890123456789012345678901234567890", "57edf4a22be3c955ac49da2e2107b67a"
    };
    int i;
    int status = 0;

    for (i = 0; i < 7*2; i += 2) {
	md5_state_t state;
	md5_byte_t digest[16];
	char hex_output[16*2 + 1];
	int di;

	md5_init(&state);
	md5_append(&state, (const md5_byte_t *)test[i], strlen(test[i]));
	md5_finish(&state, digest);
	for (di = 0; di < 16; ++di)
	    sprintf(hex_output + di * 2, "%02x", digest[di]);
	if (strcmp(hex_output, test[i + 1])) {
	    printf("MD5 (\"%s\") = ", test[i]);
	    puts(hex_output);
	    printf("**** ERROR, should be: %s\n", test[i + 1]);
	    status = 1;
	}
    }
    if (status == 0)
	puts("md5 self-test completed successfully.");
    return status;
}

/* Print the T values. */
static int
do_t_values(void)
{
    int i;
    for (i = 1; i <= 64; ++i) {
	unsigned long v = (unsigned long)(4294967296.0 * fabs(sin((double)i)));

	/*
	 * The following nonsense is only to avoid compiler warnings about
	 * "integer constant is unsigned in ANSI C, signed with -traditional".
	 */
	if (v >> 31) {
	    printf("#define T%d /* 0x%08lx */ (T_MASK ^ 0x%08lx)\n", i,
		   v, (unsigned long)(unsigned int)(~v));
	} else {
	    printf("#define T%d    0x%08lx\n", i, v);
	}
    }
    return 0;
}


/* Compute MD5 message digest for LEN bytes beginning at BUFFER.  The
   result is always in little endian byte order, so that a byte-wise
   output yields to the wanted ASCII representation of the message
   digest.  */
int md5_stream (FILE *stream, char *hex_output)
    
{
  char buffer[BLOCKSIZE + 72];
  unsigned long sum = 0, n;
  md5_state_t state;
  md5_byte_t digest[16];
  /* char hex_output[16*2 + 1];*/
  char f_temp[2+1];
  int di;

  md5_init(&state);

  /* Read block.  Take care for partial reads.  */
  do
    {
      n = fread (buffer, 1, BLOCKSIZE, stream);
      /* printf("Block = bytes=%lu total = %lu!\n", n, sum);*/
      if (n != 0)
        {
          md5_append(&state, (const md5_byte_t *)buffer, n);
          sum += n;
        }
    } while (n != 0);

  md5_finish(&state, digest);
  for (di = 0; di < 16; ++di)
     {
       sprintf(f_temp, "%02x", digest[di]);
       *(hex_output + (di * 2))     = f_temp[0];
       *(hex_output + (di * 2) + 1) = f_temp[1];
     };
  *(hex_output + (2 * 16)) = ((char) 0);
  /* puts(hex_output);*/

  return 0;
}

int recursive_catalog(char *dir_name, unsigned long *file_size_total, unsigned long *volgnummer)
{
	DIR *dir;
	struct dirent **namelist;
	int n;
	int count;
	int errnum;
	int retval;
	int count2=0;
	char path[5000];
        unsigned long number_of_rows_jm = 0;
      char hex_output[16*2 + 1];
      struct stat statbuf;
      FILE *invoer;
  
	if((dir=opendir(dir_name))==NULL)
	{
		errnum=errno;
		fprintf(stderr,"\033[1m\n%s %s:\033[31m ERROR: %s (%s)\033[0m\n",progname,dir_to_catalog,strerror(errnum),dir_name);
		return -1;
	}
	closedir(dir);
	/* fprintf(stderr,"\033[1m%s %s:\033[33m SCANNING DIR: %s\033[0m\n",progname,dir_to_catalog,dir_name);*/
	n = scandir(dir_name, &namelist, 0, alphasort);
	/* fprintf(stderr,"\033[1m%s %s:\033[32m %d entries to delete in DIR %s\033[0m\n",progname,dir_to_catalog,n-2,dir_name);*/
 //n-2 because of entries '.' and '..'
	count=1;
        /* fprintf(stderr, "%6s|%40s|%20s|%10s|%32s\n", "FileSN", "Path", "File name", "File size", "MD5");*/
	while(count<n)
	{
		if(!strcmp(namelist[count]->d_name,".") || !strcmp(namelist[count]->d_name,".."))
		{
			++count; // Skip this directory entry
			continue;
		}
		sprintf(path,"%s/%s",dir_name,namelist[count]->d_name);
		/* sprintf(text,"\033[1m%s %s:\033[33m Deleting files in DIR %s: %d of %d (%d%s)\033[0m\r",progname,dir_to_catalog,dir_name,count-1,n-2,((count-1)*100)/(n-2),"%");*/
		/* fprintf(stderr,text); */
                
		/* retval=unlink(path);*/

                if ((retval = stat(path, &statbuf)) == 0)
                  {
                     if (S_ISDIR(statbuf.st_mode))
                       {
                         /* fprintf(stderr, "Dir=%lu!\n", statbuf.st_size);*/
                         retval=recursive_catalog(path, file_size_total, volgnummer);
                       }
                     else
                       {
                         invoer = fopen(path, "r");
                         md5_stream(invoer, hex_output);
                         fclose(invoer);

                         fprintf(stderr, "%6lu|%20s|%20s|%10lu|%s\n", *volgnummer, dir_name, namelist[count]->d_name, statbuf.st_size, hex_output);

                         /* sprintf(query, "INSERT INTO catalog VALUES (%lu,'%s','%s',%lu,'%s');", *volgnummer, dir_name, namelist[count]->d_name, statbuf.st_size, hex_output);*/
 
                         sprintf(query, "SELECT FileSN, Path, FileName, FileSize, MD5 FROM catalog WHERE (FileSize=%lu) and (MD5='%s');", statbuf.st_size, hex_output);



/*****************************************************************************/
                printf("QUERY=%s!\n", query);
                number_of_rows_jm = 0;
                mysql_query(m_mysql, query);
                
                /* printf("1:Number of rows: %lu\n", (unsigned long) mysql_num_rows(result));*/

                result = mysql_use_result(m_mysql);
                /* result =mysql_list_dbs(m_mysql, "*"); */
                printf("2:Number of rows: %lu\n", (unsigned long) mysql_num_rows(result));


                if (!result) 
                  {
                     printf("Unable to execute select query");
                  }
                else
                  {
                     num_fields = mysql_num_fields(result);
                     
                     if (num_fields == 0)
                       {       
                       }
                     else
                       {
                         for (i = 0; i < num_fields; i++)
                            {
                              field = mysql_fetch_field_direct(result, i);
                              fprintf(output, "%s|", field->name);
                            }
                         fprintf(output, "\n");
                         while((row = mysql_fetch_row(result)))
                           {
                             if (row)
                               {
                                 mysql_field_seek(result, 0);
                                 num_fields = mysql_num_fields(result);
                                 for (off=0 ; off < num_fields; off++)
                                    {
                                      field = mysql_fetch_field(result);
                                      fprintf(output, "%s|", ((char*) row[off]));
                                    }
                                 fprintf(output, "\n");
                                 number_of_rows_jm++;
                               }
                           }
                         if (mysql_errno(m_mysql)) // mysql_fetch_row() failed due to an error
                           {
                             fprintf(stderr, "Error: %s\n", mysql_error(m_mysql));
                           }
                       }

                  };
                mysql_free_result(result); 
                if (number_of_rows_jm == 0)
                  {
                    printf("number_of_rows_jm=%lu!\n", number_of_rows_jm);
                   
                    sprintf(query, "INSERT INTO catalog VALUES (%lu,'%s','%s',%lu,'%s');", *volgnummer, dir_name, namelist[count]->d_name, statbuf.st_size, hex_output); 
                    printf("QUERY2=%s!\n", query);
                    mysql_query(m_mysql, query);
                
                  }
                else
                  {
                     printf("number_of_rows_jm=%lu!\n", number_of_rows_jm);
                     printf("Not inserting\n");
                     fprintf(output2,"%lu,'%s','%s',%lu,'%s'\n", *volgnummer, dir_name, namelist[count]->d_name, statbuf.st_size, hex_output);
                     file_size_not_insert =  file_size_not_insert + ((unsigned long) statbuf.st_size);
                     number_of_not_inserted_files++;
                  }
/**************************************************************************************/


                         (*volgnummer) = (*volgnummer) + 1;
                         /* fprintf(stderr, "1:File Length = %lu!retval=%d!\n", statbuf.st_size, retval);*/
                         (*file_size_total) = (*file_size_total) + statbuf.st_size;
                         /* ++count;*/
                       }
                     ++count;
                  }
                else
                  {
                    fprintf(stderr, "Couldn't stat %s\n", path);
                    count++;
                  }
            

	}
	count2=0;
	while(count2<n)
	{
		free(namelist[count2]);
		++count2;
	}
	free(namelist);
	return 0;
}

int main (int argc, char *argv[])
{
        int retval=0;
        unsigned long file_size_total = 0;
        unsigned long volgnummer = 0;

  output2 = fopen("notinsert.out", "w");
  m_mysql = mysql_init(NULL);
  if (NULL == m_mysql) 
    {
      printf("Unable to initialize connection");
    }
  else
    {
      if (NULL == mysql_real_connect(m_mysql, m_pszHost, m_pszUser, m_pszPasswd, m_pszDb,
	                               m_nPort, m_pszUnixSocket, nClientFlag)) 
        {
           printf("Unable to connect to database\n");
        }
      else
        {
           output = fopen("output.out", "w");
           if (output ==NULL)
             { 
                printf("Unable to store output\n");
             }
           else
             {
/*****************************************************************************/

        progname=argv[0];
        if(argc>1)
        {
                dir_to_catalog=argv[1];
        }
        else
        {
                fprintf(stderr,"Usage: catalog <directory>\n");
                exit(-1);
        }
        fprintf(stderr, "%6s|%20s|%20s|%10s|%32s\n", "FileSN", "Path", "File name", "File size", "MD5");
        retval=recursive_catalog(dir_to_catalog, &file_size_total, &volgnummer);
        fprintf(stderr, "Number of Files = %10lu Total file size = %15lu!\n", volgnummer, file_size_total);    

/**************************************************************************************/
                fclose(output);
             };
           mysql_close(m_mysql);
        };
    };
  fprintf(output2, "Number of not inserted files=%lu! Number of bytes in the not inserted files = %lu!\n", number_of_not_inserted_files, file_size_not_insert);

  fclose(output2);
        return 0;
}

